@@ -16,27 +16,32 @@
 #
 
 import atexit
-import datetime
 import json
 import os
 import sys
+import threading
 import time
 import traceback
-
 from datetime import datetime
 
 import azurelinuxagent.common.conf as conf
 import azurelinuxagent.common.logger as logger
-
 from azurelinuxagent.common.exception import EventError
 from azurelinuxagent.common.future import ustr
-from azurelinuxagent.common.protocol.restapi import TelemetryEventParam, \
-    TelemetryEvent, \
-    get_properties
+from azurelinuxagent.common.datacontract import get_properties
+from azurelinuxagent.common.telemetryevent import TelemetryEventParam, TelemetryEvent
 from azurelinuxagent.common.utils import fileutil, textutil
-from azurelinuxagent.common.version import CURRENT_VERSION
+from azurelinuxagent.common.version import CURRENT_VERSION, CURRENT_AGENT
 
 _EVENT_MSG = "Event: name={0}, op={1}, message={2}, duration={3}"
+TELEMETRY_EVENT_PROVIDER_ID = "69B669B9-4AF8-4C50-BDC4-6006FA76E975"
+
+# Store the last retrieved container id as an environment variable to be shared between threads for telemetry purposes
+CONTAINER_ID_ENV_VARIABLE = "AZURE_GUEST_AGENT_CONTAINER_ID"
+
+
+def get_container_id_from_env():
+    return os.environ.get(CONTAINER_ID_ENV_VARIABLE, "UNINITIALIZED")
 
 
 class WALAEventOperation:
@@ -46,6 +51,9 @@ class WALAEventOperation:
     ArtifactsProfileBlob = "ArtifactsProfileBlob"
     AutoUpdate = "AutoUpdate"
     CustomData = "CustomData"
+    CGroupsCleanUp = "CGroupsCleanUp"
+    CGroupsLimitsCrossed = "CGroupsLimitsCrossed"
+    ExtensionMetricsData = "ExtensionMetricsData"
     Deploy = "Deploy"
     Disable = "Disable"
     Downgrade = "Downgrade"
@@ -65,7 +73,9 @@ class WALAEventOperation:
     Install = "Install"
     InitializeCGroups = "InitializeCGroups"
     InitializeHostPlugin = "InitializeHostPlugin"
+    InvokeCommandUsingSystemd = "InvokeCommandUsingSystemd"
     Log = "Log"
+    OSInfo = "OSInfo"
     Partition = "Partition"
     ProcessGoalState = "ProcessGoalState"
     Provision = "Provision"
@@ -92,6 +102,7 @@ SHOULD_ENCODE_MESSAGE_OP = [
     WALAEventOperation.UnInstall,
 ]
 
+
 class EventStatus(object):
     EVENT_STATUS_FILE = "event_status.json"
 
@@ -204,7 +215,11 @@ class EventLogger(object):
             logger.warn("Cannot save event -- Event reporter is not initialized.")
             return
 
-        fileutil.mkdir(self.event_dir, mode=0o700)
+        try:
+            fileutil.mkdir(self.event_dir, mode=0o700)
+        except (IOError, OSError) as e:
+            msg = "Failed to create events folder {0}. Error: {1}".format(self.event_dir, ustr(e))
+            raise EventError(msg)
 
         existing_events = os.listdir(self.event_dir)
         if len(existing_events) >= 1000:
@@ -224,7 +239,8 @@ class EventLogger(object):
                 hfile.write(data.encode("utf-8"))
             os.rename(filename + ".tmp", filename + ".tld")
         except IOError as e:
-            raise EventError("Failed to write events to file:{0}", e)
+            msg = "Failed to write events to file: {0}".format(e)
+            raise EventError(msg)
 
     def reset_periodic(self):
         self.periodic_events = {}
@@ -233,53 +249,42 @@ class EventLogger(object):
         return h not in self.periodic_events or \
             (self.periodic_events[h] + delta) <= datetime.now()
 
-    def add_periodic(self,
-        delta, name, op=WALAEventOperation.Unknown, is_success=True, duration=0,
-        version=CURRENT_VERSION, message="", evt_type="",
-        is_internal=False, log_event=True, force=False):
+    def add_periodic(self, delta, name, op=WALAEventOperation.Unknown, is_success=True, duration=0,
+                     version=str(CURRENT_VERSION), message="", evt_type="", is_internal=False, log_event=True,
+                     force=False):
+        h = hash(name + op + ustr(is_success) + message)
 
-        h = hash(name+op+ustr(is_success)+message)
-        
         if force or self.is_period_elapsed(delta, h):
-            self.add_event(name,
-                op=op, is_success=is_success, duration=duration,
-                version=version, message=message, evt_type=evt_type,
-                is_internal=is_internal, log_event=log_event)
+            self.add_event(name, op=op, is_success=is_success, duration=duration,
+                           version=version, message=message, evt_type=evt_type,
+                           is_internal=is_internal, log_event=log_event)
             self.periodic_events[h] = datetime.now()
 
-    def add_event(self,
-                  name,
-                  op=WALAEventOperation.Unknown,
-                  is_success=True,
-                  duration=0,
-                  version=CURRENT_VERSION,
-                  message="",
-                  evt_type="",
-                  is_internal=False,
-                  log_event=True):
+    def add_event(self, name, op=WALAEventOperation.Unknown, is_success=True, duration=0, version=str(CURRENT_VERSION),
+                  message="", evt_type="", is_internal=False, log_event=True):
 
         if (not is_success) and log_event:
             _log_event(name, op, message, duration, is_success=is_success)
 
-        self._add_event(duration, evt_type, is_internal, is_success, message, name, op, version, eventId=6)
+        self._add_event(duration, evt_type, is_internal, is_success, message, name, op, version, event_id=1)
 
-    def _add_event(self, duration, evt_type, is_internal, is_success, message, name, op, version, eventId):
-        event = TelemetryEvent(eventId, "69B669B9-4AF8-4C50-BDC4-6006FA76E975")
+    def _add_event(self, duration, evt_type, is_internal, is_success, message, name, op, version, event_id):
+        event = TelemetryEvent(event_id, TELEMETRY_EVENT_PROVIDER_ID)
         event.parameters.append(TelemetryEventParam('Name', name))
         event.parameters.append(TelemetryEventParam('Version', str(version)))
         event.parameters.append(TelemetryEventParam('IsInternal', is_internal))
         event.parameters.append(TelemetryEventParam('Operation', op))
-        event.parameters.append(TelemetryEventParam('OperationSuccess',
-                                                    is_success))
+        event.parameters.append(TelemetryEventParam('OperationSuccess', is_success))
         event.parameters.append(TelemetryEventParam('Message', message))
         event.parameters.append(TelemetryEventParam('Duration', duration))
         event.parameters.append(TelemetryEventParam('ExtensionType', evt_type))
 
+        self.add_default_parameters_to_event(event)
         data = get_properties(event)
         try:
             self.save_event(json.dumps(data))
         except EventError as e:
-            logger.error("{0}", e)
+            logger.periodic_error(logger.EVERY_FIFTEEN_MINUTES, "[PERIODIC] {0}".format(ustr(e)))
 
     def add_log_event(self, level, message):
         # By the time the message has gotten to this point it is formatted as
@@ -301,6 +306,7 @@ class EventLogger(object):
         event.parameters.append(TelemetryEventParam('Context2', ''))
         event.parameters.append(TelemetryEventParam('Context3', ''))
 
+        self.add_default_parameters_to_event(event)
         data = get_properties(event)
         try:
             self.save_event(json.dumps(data))
@@ -328,12 +334,35 @@ class EventLogger(object):
         event.parameters.append(TelemetryEventParam('Instance', instance))
         event.parameters.append(TelemetryEventParam('Value', value))
 
+        self.add_default_parameters_to_event(event)
         data = get_properties(event)
         try:
             self.save_event(json.dumps(data))
         except EventError as e:
             logger.error("{0}", e)
 
+    @staticmethod
+    def add_default_parameters_to_event(event, set_default_values=False):
+        # We write the GAVersion here rather than add it in azurelinuxagent.ga.monitor.MonitorHandler.add_sysinfo
+        # as there could be a possibility of events being sent with newer version of the agent, rather than the agent
+        # version generating the event.
+        # Old behavior example: V1 writes the event on the disk and finds an update immediately, and updates. Now the
+        # new monitor thread would pick up the events from the disk and send it with the CURRENT_AGENT, which would have
+        # newer version of the agent. This causes confusion.
+        #
+        # ContainerId can change due to live migration and we want to preserve the container Id of the container writing
+        # the event, rather than sending the event.
+        # OpcodeName: This is used as the actual time of event generation.
+
+        default_parameters = [("GAVersion", CURRENT_AGENT), ('ContainerId', get_container_id_from_env()),
+                              ('OpcodeName', datetime.utcnow().__str__()),
+                              ('EventTid', threading.current_thread().ident),
+                              ('EventPid', os.getpid()), ("TaskName", threading.current_thread().getName()),
+                              ("KeywordName", '')]
+
+        for param in default_parameters:
+            event.parameters.append(TelemetryEventParam(param[0], param[1]))
+
 
 __event_logger__ = EventLogger()
 
@@ -351,7 +380,7 @@ def elapsed_milliseconds(utc_start):
 def report_event(op, is_success=True, message='', log_event=True):
     from azurelinuxagent.common.version import AGENT_NAME, CURRENT_VERSION
     add_event(AGENT_NAME,
-              version=CURRENT_VERSION,
+              version=str(CURRENT_VERSION),
               is_success=is_success,
               message=message,
               op=op,
@@ -361,10 +390,10 @@ def report_event(op, is_success=True, message='', log_event=True):
 def report_periodic(delta, op, is_success=True, message=''):
     from azurelinuxagent.common.version import AGENT_NAME, CURRENT_VERSION
     add_periodic(delta, AGENT_NAME,
-              version=CURRENT_VERSION,
-              is_success=is_success,
-              message=message,
-              op=op)
+                 version=str(CURRENT_VERSION),
+                 is_success=is_success,
+                 message=message,
+                 op=op)
 
 
 def report_metric(category, counter, instance, value, log_event=False, reporter=__event_logger__):
@@ -386,10 +415,8 @@ def report_metric(category, counter, instance, value, log_event=False, reporter=
     reporter.add_metric(category, counter, instance, value, log_event)
 
 
-def add_event(name, op=WALAEventOperation.Unknown, is_success=True, duration=0,
-              version=CURRENT_VERSION,
-              message="", evt_type="", is_internal=False, log_event=True,
-              reporter=__event_logger__):
+def add_event(name, op=WALAEventOperation.Unknown, is_success=True, duration=0, version=str(CURRENT_VERSION), message="",
+              evt_type="", is_internal=False, log_event=True, reporter=__event_logger__):
     if reporter.event_dir is None:
         logger.warn("Cannot add event -- Event reporter is not initialized.")
         _log_event(name, op, message, duration, is_success=is_success)
@@ -397,10 +424,8 @@ def add_event(name, op=WALAEventOperation.Unknown, is_success=True, duration=0,
 
     if should_emit_event(name, version, op, is_success):
         mark_event_status(name, version, op, is_success)
-        reporter.add_event(
-            name, op=op, is_success=is_success, duration=duration,
-            version=str(version), message=message, evt_type=evt_type,
-            is_internal=is_internal, log_event=log_event)
+        reporter.add_event(name, op=op, is_success=is_success, duration=duration, version=str(version), message=message,
+                           evt_type=evt_type, is_internal=is_internal, log_event=log_event)
 
 
 def add_log_event(level, message, reporter=__event_logger__):
@@ -410,20 +435,16 @@ def add_log_event(level, message, reporter=__event_logger__):
     reporter.add_log_event(level, message)
 
 
-def add_periodic(
-    delta, name, op=WALAEventOperation.Unknown, is_success=True, duration=0,
-    version=CURRENT_VERSION,
-    message="", evt_type="", is_internal=False, log_event=True, force=False,
-    reporter=__event_logger__):
+def add_periodic(delta, name, op=WALAEventOperation.Unknown, is_success=True, duration=0,
+                 version=str(CURRENT_VERSION), message="", evt_type="", is_internal=False, log_event=True, force=False,
+                 reporter=__event_logger__):
     if reporter.event_dir is None:
         logger.warn("Cannot add periodic event -- Event reporter is not initialized.")
         _log_event(name, op, message, duration, is_success=is_success)
         return
 
-    reporter.add_periodic(
-        delta, name, op=op, is_success=is_success, duration=duration,
-        version=str(version), message=message, evt_type=evt_type,
-        is_internal=is_internal, log_event=log_event, force=force)
+    reporter.add_periodic(delta, name, op=op, is_success=is_success, duration=duration, version=str(version),
+                          message=message, evt_type=evt_type, is_internal=is_internal, log_event=log_event, force=force)
 
 
 def mark_event_status(name, version, op, status):
