@@ -26,33 +26,29 @@ import random
 import re
 import shutil
 import stat
-import signal
-import subprocess
-import time
+import sys
 import tempfile
+import time
 import traceback
 import zipfile
-import sys
 
 import azurelinuxagent.common.conf as conf
 import azurelinuxagent.common.logger as logger
 import azurelinuxagent.common.utils.fileutil as fileutil
 import azurelinuxagent.common.version as version
-from azurelinuxagent.common.cgroups import CGroups, CGroupsTelemetry
+from azurelinuxagent.common.cgroupconfigurator import CGroupConfigurator
+from azurelinuxagent.common.datacontract import get_properties, set_properties
 from azurelinuxagent.common.errorstate import ErrorState, ERROR_STATE_DELTA_INSTALL
 from azurelinuxagent.common.event import add_event, WALAEventOperation, elapsed_milliseconds, report_event
 from azurelinuxagent.common.exception import ExtensionError, ProtocolError, ProtocolNotFoundError, \
-    ExtensionDownloadError, ExtensionOperationError, ExtensionErrorCodes
+    ExtensionDownloadError, ExtensionErrorCodes, ExtensionUpdateError, ExtensionOperationError
 from azurelinuxagent.common.future import ustr
 from azurelinuxagent.common.protocol import get_protocol_util
 from azurelinuxagent.common.protocol.restapi import ExtHandlerStatus, \
     ExtensionStatus, \
     ExtensionSubStatus, \
-    VMStatus, ExtHandler, \
-    get_properties, \
-    set_properties
+    VMStatus, ExtHandler
 from azurelinuxagent.common.utils.flexible_version import FlexibleVersion
-from azurelinuxagent.common.utils.processutil import format_stdout_stderr, TELEMETRY_MESSAGE_MAX_LEN
 from azurelinuxagent.common.version import AGENT_NAME, CURRENT_VERSION, GOAL_STATE_AGENT_VERSION, \
     DISTRO_NAME, DISTRO_VERSION, PY_VERSION_MAJOR, PY_VERSION_MINOR, PY_VERSION_MICRO
 
@@ -77,6 +73,21 @@ AGENT_STATUS_FILE = "waagent_status.json"
 
 NUMBER_OF_DOWNLOAD_RETRIES = 5
 
+# This is the default value for the env variables, whenever we call a command which is not an update scenario, we
+# set the env variable value to NOT_RUN to reduce ambiguity for the extension publishers
+NOT_RUN = "NOT_RUN"
+
+
+class ExtCommandEnvVariable(object):
+    Prefix = "AZURE_GUEST_AGENT"
+    DisableReturnCode = "%s_DISABLE_CMD_EXIT_CODE" % Prefix
+    UninstallReturnCode = "%s_UNINSTALL_CMD_EXIT_CODE" % Prefix
+    ExtensionPath = "%s_EXTENSION_PATH" % Prefix
+    ExtensionVersion = "%s_EXTENSION_VERSION" % Prefix
+    ExtensionSeqNumber = "ConfigSequenceNumber"  # At par with Windows Guest Agent
+    UpdatingFromVersion = "%s_UPDATING_FROM_VERSION" % Prefix
+
+
 def get_traceback(e):
     if sys.version_info[0] == 3:
         return e.__traceback__
@@ -247,8 +258,10 @@ class ExtHandlersHandler(object):
             logger.verbose(msg)
             # Log status report success on new config
             self.log_report = True
-            self.handle_ext_handlers(etag)
-            self.last_etag = etag
+
+            if self.extension_processing_allowed():
+                self.handle_ext_handlers(etag)
+                self.last_etag = etag
 
             self.report_ext_handlers_status()
             self.cleanup_outdated_handlers()
@@ -312,7 +325,7 @@ class ExtHandlersHandler(object):
         # Finally, remove the directories and packages of the
         # uninstalled handlers
         for handler in handlers:
-            handler.rm_ext_handler_dir()
+            handler.remove_ext_handler()
             pkg = os.path.join(conf.get_lib_dir(), handler.get_full_name() + HANDLER_PKG_EXT)
             if os.path.isfile(pkg):
                 try:
@@ -321,15 +334,10 @@ class ExtHandlersHandler(object):
                 except OSError as e:
                     logger.warn("Failed to remove extension package {0}: {1}".format(pkg, e.strerror))
 
-    def handle_ext_handlers(self, etag=None):
+    def extension_processing_allowed(self):
         if not conf.get_extensions_enabled():
             logger.verbose("Extension handling is disabled")
-            return
-
-        if self.ext_handlers.extHandlers is None or \
-                len(self.ext_handlers.extHandlers) == 0:
-            logger.verbose("No extension handler config found")
-            return
+            return False
 
         if conf.get_enable_overprovisioning():
             if not self.protocol.supports_overprovisioning():
@@ -338,7 +346,15 @@ class ExtHandlersHandler(object):
                 artifacts_profile = self.protocol.get_artifacts_profile()
                 if artifacts_profile and artifacts_profile.is_on_hold():
                     logger.info("Extension handling is on hold")
-                    return
+                    return False
+
+        return True
+
+    def handle_ext_handlers(self, etag=None):
+        if self.ext_handlers.extHandlers is None or \
+                len(self.ext_handlers.extHandlers) == 0:
+            logger.verbose("No extension handler config found")
+            return
 
         wait_until = datetime.datetime.utcnow() + datetime.timedelta(minutes=DEFAULT_EXT_TIMEOUT_MINUTES)
         max_dep_level = max([handler.sort_key() for handler in self.ext_handlers.extHandlers])
@@ -412,7 +428,7 @@ class ExtHandlersHandler(object):
                 return
 
             self.get_artifact_error_state.reset()
-            if not ext_handler_i.is_upgrade and self.last_etag == etag:
+            if self.last_etag == etag:
                 if self.log_etag:
                     ext_handler_i.logger.verbose("Version {0} is current for etag {1}",
                                                  ext_handler_i.pkg.version,
@@ -432,20 +448,37 @@ class ExtHandlersHandler(object):
             else:
                 message = u"Unknown ext handler state:{0}".format(state)
                 raise ExtensionError(message)
-        except ExtensionOperationError as e:
-            self.handle_handle_ext_handler_error(ext_handler_i, e, e.code)
+        except ExtensionUpdateError as e:
+            # Not reporting the error as it has already been reported from the old version
+            self.handle_ext_handler_error(ext_handler_i, e, e.code, report_telemetry_event=False)
+        except ExtensionDownloadError as e:
+            self.handle_ext_handler_download_error(ext_handler_i, e, e.code)
         except ExtensionError as e:
-            self.handle_handle_ext_handler_error(ext_handler_i, e, e.code)
+            self.handle_ext_handler_error(ext_handler_i, e, e.code)
         except Exception as e:
-            self.handle_handle_ext_handler_error(ext_handler_i, e)
+            self.handle_ext_handler_error(ext_handler_i, e)
 
-    def handle_handle_ext_handler_error(self, ext_handler_i, e, code=-1):
+    def handle_ext_handler_error(self, ext_handler_i, e, code=-1, report_telemetry_event=True):
         msg = ustr(e)
         ext_handler_i.set_handler_status(message=msg, code=code)
-        ext_handler_i.report_event(message=msg, is_success=False, log_event=True)
+
+        if report_telemetry_event:
+            ext_handler_i.report_event(message=msg, is_success=False, log_event=True)
+
+    def handle_ext_handler_download_error(self, ext_handler_i, e, code=-1):
+        msg = ustr(e)
+        ext_handler_i.set_handler_status(message=msg, code=code)
+
+        self.get_artifact_error_state.incr()
+        if self.get_artifact_error_state.is_triggered():
+            report_event(op=WALAEventOperation.Download, is_success=False, log_event=True,
+                         message="Failed to get artifact for over "
+                                 "{0}: {1}".format(self.get_artifact_error_state.min_timedelta, msg))
+            self.get_artifact_error_state.reset()
 
     def handle_enable(self, ext_handler_i):
         self.log_process = True
+        uninstall_exit_code = None
         old_ext_handler_i = ext_handler_i.get_installed_ext_handler()
 
         handler_state = ext_handler_i.get_handler_state()
@@ -459,19 +492,58 @@ class ExtHandlersHandler(object):
             if old_ext_handler_i is None:
                 ext_handler_i.install()
             elif ext_handler_i.version_ne(old_ext_handler_i):
-                old_ext_handler_i.disable()
-                ext_handler_i.copy_status_files(old_ext_handler_i)
-                if ext_handler_i.version_gt(old_ext_handler_i):
-                    ext_handler_i.update()
-                else:
-                    old_ext_handler_i.update(version=ext_handler_i.ext_handler.properties.version)
-                old_ext_handler_i.uninstall()
-                old_ext_handler_i.rm_ext_handler_dir()
-                ext_handler_i.update_with_install()
+                uninstall_exit_code = ExtHandlersHandler._update_extension_handler_and_return_if_failed(
+                    old_ext_handler_i, ext_handler_i)
         else:
             ext_handler_i.update_settings()
 
-        ext_handler_i.enable()
+        ext_handler_i.enable(uninstall_exit_code=uninstall_exit_code)
+
+    @staticmethod
+    def _update_extension_handler_and_return_if_failed(old_ext_handler_i, ext_handler_i):
+
+        def execute_old_handler_command_and_return_if_succeeds(func):
+            """
+            Created a common wrapper to execute all commands that need to be executed from the old handler
+            so that it can have a common exception handling mechanism
+            :param func: The command to be executed on the old handler
+            :return: True if command execution succeeds and False if it fails
+            """
+            continue_on_update_failure = False
+            exit_code = 0
+            try:
+                continue_on_update_failure = ext_handler_i.load_manifest().is_continue_on_update_failure()
+                func()
+            except ExtensionError as e:
+                # Reporting the event with the old handler and raising a new ExtensionUpdateError to set the
+                # handler status on the new version
+                msg = "%s; ContinueOnUpdate: %s" % (ustr(e), continue_on_update_failure)
+                old_ext_handler_i.report_event(message=msg, is_success=False)
+                if not continue_on_update_failure:
+                    raise ExtensionUpdateError(msg)
+
+                exit_code = e.code
+                if isinstance(e, ExtensionOperationError):
+                    exit_code = e.exit_code
+
+                logger.info("Continue on Update failure flag is set, proceeding with update")
+            return exit_code
+
+        disable_exit_code = execute_old_handler_command_and_return_if_succeeds(
+            func=lambda: old_ext_handler_i.disable())
+        ext_handler_i.copy_status_files(old_ext_handler_i)
+        if ext_handler_i.version_gt(old_ext_handler_i):
+            ext_handler_i.update(disable_exit_code=disable_exit_code,
+                                 updating_from_version=old_ext_handler_i.ext_handler.properties.version)
+        else:
+            updating_from_version = ext_handler_i.ext_handler.properties.version
+            old_ext_handler_i.update(version=updating_from_version,
+                                     disable_exit_code=disable_exit_code, updating_from_version=updating_from_version)
+        uninstall_exit_code = execute_old_handler_command_and_return_if_succeeds(
+            func=lambda: old_ext_handler_i.uninstall())
+        old_ext_handler_i.remove_ext_handler()
+        ext_handler_i.update_with_install(uninstall_exit_code=uninstall_exit_code)
+        return uninstall_exit_code
 
     def handle_disable(self, ext_handler_i):
         self.log_process = True
@@ -489,8 +561,14 @@ class ExtHandlersHandler(object):
         if handler_state != ExtHandlerState.NotInstalled:
             if handler_state == ExtHandlerState.Enabled:
                 ext_handler_i.disable()
-            ext_handler_i.uninstall()
-        ext_handler_i.rm_ext_handler_dir()
+
+            # Try uninstalling the extension and swallow any exceptions in case of failures after logging them
+            try:
+                ext_handler_i.uninstall()
+            except ExtensionError as e:
+                ext_handler_i.report_event(message=ustr(e), is_success=False)
+
+        ext_handler_i.remove_ext_handler()
 
     def report_ext_handlers_status(self):
         """
@@ -606,7 +684,6 @@ class ExtHandlerInstance(object):
         self.operation = None
         self.pkg = None
         self.pkg_file = None
-        self.is_upgrade = False
         self.logger = None
         self.set_logger()
 
@@ -625,6 +702,9 @@ class ExtHandlerInstance(object):
             pkg_list = self.protocol.get_ext_handler_pkgs(self.ext_handler)
         except ProtocolError as e:
             raise ExtensionError("Failed to get ext handler pkgs", e)
+        except ExtensionDownloadError:
+            self.set_operation(WALAEventOperation.Download)
+            raise
 
         # Determine the desired and installed versions
         requested_version = FlexibleVersion(str(self.ext_handler.properties.version))
@@ -664,12 +744,6 @@ class ExtHandlerInstance(object):
             if self.pkg is not None:
                 self.ext_handler.properties.version = str(selected_pkg.version)
 
-        # Note if the selected package is different than that installed
-        if installed_pkg is None \
-                or (
-                self.pkg is not None and FlexibleVersion(self.pkg.version) != FlexibleVersion(installed_pkg.version)):
-            self.is_upgrade = True
-
         if self.pkg is not None:
             self.logger.verbose("Use version: {0}", self.pkg.version)
         self.set_logger()
@@ -779,7 +853,7 @@ class ExtHandlerInstance(object):
         if self.pkg is None or self.pkg.uris is None or len(self.pkg.uris) == 0:
             raise ExtensionDownloadError("No package uri found")
 
-        destination = os.path.join(conf.get_lib_dir(), os.path.basename(self.pkg.uris[0].uri) + ".zip")
+        destination = os.path.join(conf.get_lib_dir(), self.get_extension_package_zipfile_name())
 
         package_exists = False
         if os.path.exists(destination):
@@ -815,13 +889,13 @@ class ExtHandlerInstance(object):
                 raise ExtensionDownloadError("Failed to download extension",
                                              code=ExtensionErrorCodes.PluginManifestDownloadError)
 
+            duration = elapsed_milliseconds(begin_utc)
+            self.report_event(message="Download succeeded", duration=duration)
+
         self.pkg_file = destination
 
-        duration = elapsed_milliseconds(begin_utc)
-        self.report_event(message="Download succeeded", duration=duration)
-
     def initialize(self):
-        self.logger.info("Initialize extension directory")
+        self.logger.info("Initializing extension {0}".format(self.get_full_name()))
 
         # Add user execute permission to all files under the base dir
         for file in fileutil.get_all_files(self.get_base_dir()):
@@ -865,18 +939,24 @@ class ExtHandlerInstance(object):
 
         except IOError as e:
             fileutil.clean_ioerror(e, paths=[self.get_base_dir(), self.pkg_file])
-            raise ExtensionDownloadError(u"Failed to create status or config dir", e)
+            raise ExtensionDownloadError(u"Failed to initialize extension '{0}'".format(self.get_full_name()), e)
+
+        # Create cgroups for the extension
+        CGroupConfigurator.get_instance().create_extension_cgroups(self.get_full_name())
 
         # Save HandlerEnvironment.json
         self.create_handler_env()
 
-    def enable(self):
+    def enable(self, uninstall_exit_code=None):
+        uninstall_exit_code = str(uninstall_exit_code) if uninstall_exit_code is not None else NOT_RUN
+        env = {ExtCommandEnvVariable.UninstallReturnCode: uninstall_exit_code}
+
         self.set_operation(WALAEventOperation.Enable)
         man = self.load_manifest()
         enable_cmd = man.get_enable_command()
         self.logger.info("Enable extension [{0}]".format(enable_cmd))
         self.launch_command(enable_cmd, timeout=300,
-                            extension_error_code=ExtensionErrorCodes.PluginEnableProcessingFailed)
+                            extension_error_code=ExtensionErrorCodes.PluginEnableProcessingFailed, env=env)
         self.set_handler_state(ExtHandlerState.Enabled)
         self.set_handler_status(status="Ready", message="Plugin enabled")
 
@@ -890,47 +970,59 @@ class ExtHandlerInstance(object):
         self.set_handler_state(ExtHandlerState.Installed)
         self.set_handler_status(status="NotReady", message="Plugin disabled")
 
-    def install(self):
+    def install(self, uninstall_exit_code=None):
+        uninstall_exit_code = str(uninstall_exit_code) if uninstall_exit_code is not None else NOT_RUN
+        env = {ExtCommandEnvVariable.UninstallReturnCode: uninstall_exit_code}
+
         man = self.load_manifest()
         install_cmd = man.get_install_command()
         self.logger.info("Install extension [{0}]".format(install_cmd))
         self.set_operation(WALAEventOperation.Install)
         self.launch_command(install_cmd, timeout=900,
-                            extension_error_code=ExtensionErrorCodes.PluginInstallProcessingFailed)
+                            extension_error_code=ExtensionErrorCodes.PluginInstallProcessingFailed, env=env)
         self.set_handler_state(ExtHandlerState.Installed)
 
     def uninstall(self):
-        try:
-            self.set_operation(WALAEventOperation.UnInstall)
-            man = self.load_manifest()
-            uninstall_cmd = man.get_uninstall_command()
-            self.logger.info("Uninstall extension [{0}]".format(uninstall_cmd))
-            self.launch_command(uninstall_cmd)
-        except ExtensionError as e:
-            self.report_event(message=ustr(e), is_success=False)
+        self.set_operation(WALAEventOperation.UnInstall)
+        man = self.load_manifest()
+        uninstall_cmd = man.get_uninstall_command()
+        self.logger.info("Uninstall extension [{0}]".format(uninstall_cmd))
+        self.launch_command(uninstall_cmd)
 
-    def rm_ext_handler_dir(self):
+    def remove_ext_handler(self):
         try:
-            zip_filename = "__".join(os.path.basename(self.get_base_dir()).split("-")) + ".zip"
-            destination = os.path.join(conf.get_lib_dir(), zip_filename)
-            if os.path.exists(destination):
-                self.pkg_file = destination
-                os.remove(self.pkg_file)
+            zip_filename = os.path.join(conf.get_lib_dir(), self.get_extension_package_zipfile_name())
+            if os.path.exists(zip_filename):
+                os.remove(zip_filename)
+                self.logger.verbose("Deleted the extension zip at path {0}", zip_filename)
 
             base_dir = self.get_base_dir()
             if os.path.isdir(base_dir):
-                self.logger.info("Remove extension handler directory: {0}",
-                                 base_dir)
-                shutil.rmtree(base_dir)
+                self.logger.info("Remove extension handler directory: {0}", base_dir)
+
+                # some extensions uninstall asynchronously so ignore error 2 while removing them
+                def on_rmtree_error(_, __, exc_info):
+                    _, exception, _ = exc_info
+                    if not isinstance(exception, OSError) or exception.errno != 2:  # [Errno 2] No such file or directory
+                        raise exception
+
+                shutil.rmtree(base_dir, onerror=on_rmtree_error)
         except IOError as e:
             message = "Failed to remove extension handler directory: {0}".format(e)
             self.report_event(message=message, is_success=False)
             self.logger.warn(message)
 
-    def update(self, version=None):
+        # Also remove the cgroups for the extension
+        CGroupConfigurator.get_instance().remove_extension_cgroups(self.get_full_name())
+
+    def update(self, version=None, disable_exit_code=None, updating_from_version=None):
         if version is None:
             version = self.ext_handler.properties.version
 
+        disable_exit_code = str(disable_exit_code) if disable_exit_code is not None else NOT_RUN
+        env = {'VERSION': version, ExtCommandEnvVariable.DisableReturnCode: disable_exit_code,
+               ExtCommandEnvVariable.UpdatingFromVersion: updating_from_version}
+
         try:
             self.set_operation(WALAEventOperation.Update)
             man = self.load_manifest()
@@ -939,16 +1031,16 @@ class ExtHandlerInstance(object):
             self.launch_command(update_cmd,
                                 timeout=900,
                                 extension_error_code=ExtensionErrorCodes.PluginUpdateProcessingFailed,
-                                env={'VERSION': version})
+                                env=env)
         except ExtensionError:
             # prevent the handler update from being retried
             self.set_handler_state(ExtHandlerState.Failed)
             raise
 
-    def update_with_install(self):
+    def update_with_install(self, uninstall_exit_code=None):
         man = self.load_manifest()
         if man.is_update_with_install():
-            self.install()
+            self.install(uninstall_exit_code=uninstall_exit_code)
         else:
             self.logger.info("UpdateWithInstall not set. "
                              "Skip install during upgrade.")
@@ -1105,7 +1197,8 @@ class ExtHandlerInstance(object):
         last_update = int(time.time() - os.stat(heartbeat_file).st_mtime)
         return last_update <= 600
 
-    def launch_command(self, cmd, timeout=300, extension_error_code=ExtensionErrorCodes.PluginProcessingError, env=None):
+    def launch_command(self, cmd, timeout=300, extension_error_code=ExtensionErrorCodes.PluginProcessingError,
+                       env=None):
         begin_utc = datetime.datetime.utcnow()
         self.logger.verbose("Launch command: [{0}]", cmd)
 
@@ -1116,80 +1209,38 @@ class ExtHandlerInstance(object):
                 if env is None:
                     env = {}
                 env.update(os.environ)
+                # Always add Extension Path and version to the current launch_command (Ask from publishers)
+                env.update({ExtCommandEnvVariable.ExtensionPath: base_dir,
+                            ExtCommandEnvVariable.ExtensionVersion: str(self.ext_handler.properties.version),
+                            ExtCommandEnvVariable.ExtensionSeqNumber: str(self.get_seq_no())})
 
                 try:
                     # Some extensions erroneously begin cmd with a slash; don't interpret those
                     # as root-relative. (Issue #1170)
                     full_path = os.path.join(base_dir, cmd.lstrip(os.path.sep))
 
-                    def pre_exec_function():
-                        """
-                        Change process state before the actual target process is started. Effectively, this runs between
-                        the fork() and the exec() of sub-process creation.
-                        """
-                        os.setsid()
-                        CGroups.add_to_extension_cgroup(self.ext_handler.name, os.getpid())
+                    process_output = CGroupConfigurator.get_instance().start_extension_command(
+                        extension_name=self.get_full_name(),
+                        command=full_path,
+                        timeout=timeout,
+                        shell=True,
+                        cwd=base_dir,
+                        env=env,
+                        stdout=stdout,
+                        stderr=stderr,
+                        error_code=extension_error_code)
 
-                    process = subprocess.Popen(full_path,
-                                               shell=True,
-                                               cwd=base_dir,
-                                               stdout=stdout,
-                                               stderr=stderr,
-                                               env=env,
-                                               preexec_fn=pre_exec_function)
                 except OSError as e:
-                    raise ExtensionOperationError("Failed to launch '{0}': {1}".format(full_path, e.strerror),
-                                                  code=extension_error_code)
-
-                try:
-                    cg = CGroups.for_extension(self.ext_handler.name)
-                    CGroupsTelemetry.track_extension(self.ext_handler.name, cg)
-                except Exception as e:
-                    self.logger.warn("Unable to setup cgroup {0}: {1}".format(self.ext_handler.name, e))
-
-                msg = ExtHandlerInstance._capture_process_output(process, stdout, stderr, cmd, timeout, extension_error_code)
+                    raise ExtensionError("Failed to launch '{0}': {1}".format(full_path, e.strerror),
+                                         code=extension_error_code)
 
                 duration = elapsed_milliseconds(begin_utc)
-                log_msg = "{0}\n{1}".format(cmd, "\n".join([line for line in msg.split('\n') if line != ""]))
+                log_msg = "{0}\n{1}".format(cmd, "\n".join([line for line in process_output.split('\n') if line != ""]))
+
                 self.logger.verbose(log_msg)
                 self.report_event(message=log_msg, duration=duration, log_event=False)
 
-                return msg
-
-    @staticmethod
-    def _capture_process_output(process, stdout_file, stderr_file, cmd, timeout,
-                                code=ExtensionErrorCodes.PluginUnknownFailure):
-        retry = timeout
-        while retry > 0 and process.poll() is None:
-            time.sleep(1)
-            retry -= 1
-
-        def read_output():
-            try:
-                stdout_file.seek(0)
-                stderr_file.seek(0)
-
-                stdout = ustr(stdout_file.read(TELEMETRY_MESSAGE_MAX_LEN), encoding='utf-8', errors='backslashreplace')
-                stderr = ustr(stderr_file.read(TELEMETRY_MESSAGE_MAX_LEN), encoding='utf-8', errors='backslashreplace')
-
-                return format_stdout_stderr(stdout, stderr)
-            except Exception as e:
-                return format_stdout_stderr("", "Cannot read stdout/stderr: {0}".format(str(e)))
-
-        # timeout expired
-        if retry == 0:
-            os.killpg(os.getpgid(process.pid), signal.SIGKILL)
-            raise ExtensionError("Timeout({0}): {1}\n{2}".format(timeout, cmd, read_output()),
-                                 code=ExtensionErrorCodes.PluginHandlerScriptTimedout)
-
-        # process completed or forked; sleep 1 sec to give the child process (if any) a chance to start
-        time.sleep(1)
-
-        return_code = process.wait()
-        if return_code != 0:
-            raise ExtensionError("Non-zero exit code: {0}, {1}\n{2}".format(return_code, cmd, read_output()), code=code)
-
-        return read_output()
+                return process_output
 
     def load_manifest(self):
         man_file = self.get_manifest_file()
@@ -1315,6 +1366,11 @@ class ExtHandlerInstance(object):
         except (IOError, ValueError) as e:
             self.logger.error("Failed to get handler status: {0}", e)
 
+    def get_extension_package_zipfile_name(self):
+        return "{0}__{1}{2}".format(self.ext_handler.name,
+                                    self.ext_handler.properties.version,
+                                    HANDLER_PKG_EXT)
+
     def get_full_name(self):
         return "{0}-{1}".format(self.ext_handler.name,
                                 self.ext_handler.properties.version)
@@ -1340,6 +1396,17 @@ class ExtHandlerInstance(object):
     def get_log_dir(self):
         return os.path.join(conf.get_ext_log_dir(), self.ext_handler.name)
 
+    def get_seq_no(self):
+        runtime_settings = self.ext_handler.properties.extensions
+        # If no runtime_settings available for this ext_handler, then return 0 (this is the behavior we follow
+        # for update_settings)
+        if not runtime_settings or len(runtime_settings) == 0:
+            return "0"
+        # Currently for every runtime settings we use the same sequence number
+        # (Check : def parse_plugin_settings(self, ext_handler, plugin_settings) in wire.py)
+        # Will have to revisit once the feature to enable multiple runtime settings is rolled out by CRP
+        return self.ext_handler.properties.extensions[0].sequenceNumber
+
 
 class HandlerEnvironment(object):
     def __init__(self, data):
@@ -1396,3 +1463,6 @@ class HandlerManifest(object):
         if update_mode is None:
             return True
         return update_mode.lower() == "updatewithinstall"
+
+    def is_continue_on_update_failure(self):
+        return self.data['handlerManifest'].get('continueOnUpdateFailure', False)
