@@ -1,15 +1,14 @@
 # Copyright (c) Microsoft Corporation. All rights reserved.
 # Licensed under the Apache License.
 import json
-import re
-import stat
+import subprocess
 
 from azurelinuxagent.common.protocol.restapi import ExtensionStatus, Extension, ExtHandler, ExtHandlerProperties
-from azurelinuxagent.ga.exthandlers import parse_ext_status, ExtHandlerInstance, get_exthandlers_handler
+from azurelinuxagent.ga.exthandlers import parse_ext_status, ExtHandlerInstance, get_exthandlers_handler, \
+    ExtCommandEnvVariable
 from azurelinuxagent.common.exception import ProtocolError, ExtensionError, ExtensionErrorCodes
 from azurelinuxagent.common.event import WALAEventOperation
-from azurelinuxagent.common.utils.processutil import TELEMETRY_MESSAGE_MAX_LEN, format_stdout_stderr
-from azurelinuxagent.common.cgroups import CGroups
+from azurelinuxagent.common.utils.extensionprocessutil import TELEMETRY_MESSAGE_MAX_LEN, format_stdout_stderr, read_output
 from tests.tools import *
 
 
@@ -227,46 +226,31 @@ class LaunchCommandTestCase(AgentTestCase):
         self.ext_handler.properties = ext_handler_properties
         self.ext_handler_instance = ExtHandlerInstance(ext_handler=self.ext_handler, protocol=None)
 
-        self.base_cgroups = os.path.join(self.tmp_dir, "cgroup")
-        os.mkdir(self.base_cgroups)
-        os.mkdir(os.path.join(self.base_cgroups, "cpu"))
-        os.mkdir(os.path.join(self.base_cgroups, "memory"))
-
-        self.mock__base_cgroups = patch("azurelinuxagent.common.cgroups.BASE_CGROUPS", self.base_cgroups)
-        self.mock__base_cgroups.start()
-
         self.mock_get_base_dir = patch("azurelinuxagent.ga.exthandlers.ExtHandlerInstance.get_base_dir", lambda *_: self.tmp_dir)
         self.mock_get_base_dir.start()
 
-        log_dir = os.path.join(self.tmp_dir, "log")
+        self.log_dir = os.path.join(self.tmp_dir, "log")
         self.mock_get_log_dir = patch("azurelinuxagent.ga.exthandlers.ExtHandlerInstance.get_log_dir", lambda *_: self.log_dir)
         self.mock_get_log_dir.start()
 
+        self.mock_sleep = patch("time.sleep", lambda *_: mock_sleep(0.01))
+        self.mock_sleep.start()
+
+        self.cgroups_enabled = CGroupConfigurator.get_instance().enabled()
+        CGroupConfigurator.get_instance().disable()
+
     def tearDown(self):
+        if self.cgroups_enabled:
+            CGroupConfigurator.get_instance().enable()
+        else:
+            CGroupConfigurator.get_instance().disable()
+
         self.mock_get_log_dir.stop()
         self.mock_get_base_dir.stop()
-        self.mock__base_cgroups.stop()
+        self.mock_sleep.stop()
 
         AgentTestCase.tearDown(self)
 
-    def _create_script(self, file_name, contents):
-        """
-        Creates an executable script with the given contents.
-        If file_name ends with ".py", it creates a Python3 script, otherwise it creates a bash script
-        """
-        file_path = os.path.join(self.ext_handler_instance.get_base_dir(), file_name)
-
-        with open(file_path, "w") as script:
-            if file_name.endswith(".py"):
-                script.write("#!/usr/bin/env python3\n")
-            else:
-                script.write("#!/usr/bin/env bash\n")
-            script.write(contents)
-
-        os.chmod(file_path, stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR)
-
-        return file_name
-
     @staticmethod
     def _output_regex(stdout, stderr):
         return r"\[stdout\]\s+{0}\s+\[stderr\]\s+{1}".format(stdout, stderr)
@@ -287,7 +271,7 @@ class LaunchCommandTestCase(AgentTestCase):
         stdout = "stdout" * 5
         stderr = "stderr" * 5
 
-        command = self._create_script("produce_output.py", '''
+        command = self.create_script("produce_output.py", '''
 import sys
 
 sys.stdout.write("{0}")
@@ -318,7 +302,7 @@ sys.stderr.write("{1}")
         signal_file = os.path.join(self.tmp_dir, "signal_file.txt")
 
         # the test command produces some output then goes into an infinite loop
-        command = self._create_script("produce_output_then_hang.py", '''
+        command = self.create_script("produce_output_then_hang.py", '''
 import sys
 import time
 
@@ -353,7 +337,8 @@ with open("{2}", "w") as file:
 
             # the command name and its output should be part of the message
             message = str(context_manager.exception)
-            self.assertRegex(message, r"Timeout\(\d+\):\s+{0}\s+{1}".format(command, LaunchCommandTestCase._output_regex(stdout, stderr)))
+            command_full_path = os.path.join(self.tmp_dir, command.lstrip(os.path.sep))
+            self.assertRegex(message, r"Timeout\(\d+\):\s+{0}\s+{1}".format(command_full_path, LaunchCommandTestCase._output_regex(stdout, stderr)))
 
             # the exception code should be as specified in the call to launch_command
             self.assertEquals(context_manager.exception.code, extension_error_code)
@@ -373,7 +358,7 @@ with open("{2}", "w") as file:
         stderr = "stderr" * 3
         exit_code = 99
 
-        command = self._create_script("fail.py", '''
+        command = self.create_script("fail.py", '''
 import sys
 
 sys.stdout.write("{0}")
@@ -395,7 +380,7 @@ exit({2})
         stdout = "stdout"
         stderr = "stderr"
 
-        command = self._create_script("start_child_process.py", '''
+        command = self.create_script("start_child_process.py", '''
 import os
 import sys
 import time
@@ -430,7 +415,7 @@ else:
         # the child process uses the signal file to indicate it has produced output
         signal_file = os.path.join(self.tmp_dir, "signal_file.txt")
 
-        command = self._create_script("start_child_with_output.py", '''
+        command = self.create_script("start_child_with_output.py", '''
 import os
 import sys
 import time
@@ -471,7 +456,7 @@ else:
         child_stdout = "CHILD STDOUT"
         child_stderr = "CHILD STDERR"
 
-        command = self._create_script("start_child_that_fails.py", '''
+        command = self.create_script("start_child_that_fails.py", '''
 import os
 import sys
 import time
@@ -500,7 +485,7 @@ else:
         # file used to verify the command completed successfully
         signal_file = os.path.join(self.tmp_dir, "signal_file.txt")
 
-        command = self._create_script("create_file.py", '''
+        command = self.create_script("create_file.py", '''
 open("{0}", "w").close()
 
 '''.format(signal_file))
@@ -517,7 +502,7 @@ open("{0}", "w").close()
         stderr = "STDERR"
 
         # the test script mimics the redirection done by the Custom Script extension
-        command = self._create_script("produce_output", '''
+        command = self.create_script("produce_output", '''
 exec &> {0}
 echo {1}
 >&2 echo {2}
@@ -536,7 +521,7 @@ echo {1}
         stdout = "STDOUT"
         stderr = "STDERR"
 
-        command = self._create_script("produce_long_output.py", '''
+        command = self.create_script("produce_long_output.py", '''
 import sys
 
 sys.stdout.write( "{0}" * {1})
@@ -550,7 +535,7 @@ sys.stderr.write( "{2}" * {3})
         self.assertIn(stderr, output)
 
     def test_it_should_read_only_the_head_of_large_outputs(self):
-        command = self._create_script("produce_long_output.py", '''
+        command = self.create_script("produce_long_output.py", '''
 import sys
 
 sys.stdout.write("O" * 5 * 1024 * 1024)
@@ -560,7 +545,7 @@ sys.stderr.write("E" * 5 * 1024 * 1024)
         # Mocking the call to file.read() is difficult, so instead we mock the call to format_stdout_stderr, which takes the
         # return value of the calls to file.read(). The intention of the test is to verify we never read (and load in memory)
         # more than a few KB of data from the files used to capture stdout/stderr
-        with patch('azurelinuxagent.ga.exthandlers.format_stdout_stderr', side_effect=format_stdout_stderr) as mock_format:
+        with patch('azurelinuxagent.common.utils.extensionprocessutil.format_stdout_stderr', side_effect=format_stdout_stderr) as mock_format:
             output = self.ext_handler_instance.launch_command(command)
 
         self.assertGreaterEqual(len(output), 1024)
@@ -578,67 +563,46 @@ sys.stderr.write("E" * 5 * 1024 * 1024)
         self.assertLessEqual(len(stderr), TELEMETRY_MESSAGE_MAX_LEN)
 
     def test_it_should_handle_errors_while_reading_the_command_output(self):
-        command = self._create_script("produce_output.py", '''
+        command = self.create_script("produce_output.py", '''
 import sys
 
 sys.stdout.write("STDOUT")
 sys.stderr.write("STDERR")
 ''')
+        # Mocking the call to file.read() is difficult, so instead we mock the call to_capture_process_output,
+        # which will call file.read() and we force stdout/stderr to be None; this will produce an exception when
+        # trying to use these files.
+        original_capture_process_output = read_output
 
-        # Mocking the call to file.read() is difficult, so instead we mock the call to _capture_process_output, which will
-        # call file.read() and we force stdout/stderr to be None; this will produce an exception when trying to use these files.
-        original_capture_process_output = ExtHandlerInstance._capture_process_output
+        def capture_process_output(stdout_file, stderr_file):
+            return original_capture_process_output(None, None)
 
-        def capture_process_output(process, stdout_file, stderr_file, cmd, timeout, code):
-            return original_capture_process_output(process, None, None, cmd, timeout, code)
-
-        with patch('azurelinuxagent.ga.exthandlers.ExtHandlerInstance._capture_process_output', side_effect=capture_process_output):
+        with patch('azurelinuxagent.common.utils.extensionprocessutil.read_output', side_effect=capture_process_output):
             output = self.ext_handler_instance.launch_command(command)
 
         self.assertIn("[stderr]\nCannot read stdout/stderr:", output)
 
-    def test_it_should_handle_exceptions_from_cgroups_and_run_command(self):
-        # file used to verify the command completed successfully
-        signal_file = os.path.join(self.tmp_dir, "signal_file.txt")
+    def test_it_should_contain_all_helper_environment_variables(self):
 
-        command = self._create_script("create_file.py", '''
-open("{0}", "w").close()
+        helper_env_vars = {ExtCommandEnvVariable.ExtensionSeqNumber: self.ext_handler_instance.get_seq_no(),
+                           ExtCommandEnvVariable.ExtensionPath: self.tmp_dir,
+                           ExtCommandEnvVariable.ExtensionVersion: self.ext_handler_instance.ext_handler.properties.version}
 
-'''.format(signal_file))
+        command = """
+            printenv | grep -E '(%s)'
+        """ % '|'.join(helper_env_vars.keys())
 
-        with patch('azurelinuxagent.common.cgroups.CGroups.for_extension', side_effect=Exception):
-            self.ext_handler_instance.launch_command(command)
+        test_file = self.create_script('printHelperEnvironments.sh', command)
 
-        self.assertTrue(os.path.exists(signal_file))
+        with patch("subprocess.Popen", wraps=subprocess.Popen) as patch_popen:
+            output = self.ext_handler_instance.launch_command(test_file)
 
-    @skip_if_predicate_false(CGroups.enabled, "CGroups not supported in this environment")
-    def test_it_should_add_the_child_process_to_its_own_cgroup(self):
-        # We are checking for the parent PID here since the PID getting written to the corresponding cgroup
-        # would be from the shell process started before launch_command invokes the actual command.
-        # In a non-mocked scenario, the kernel would actually also write all the children's PIDs to the procs
-        # file as well, but here we are mocking the base cgroup path, so it is not taken care for us.
-        command = self._create_script("output_parent_pid.py", '''
-import os
+            args, kwagrs = patch_popen.call_args
+            without_os_env = dict((k, v) for (k, v) in kwagrs['env'].items() if k not in os.environ)
 
-print(os.getppid())
-
-''')
-
-        output = self.ext_handler_instance.launch_command(command)
-
-        match = re.match(LaunchCommandTestCase._output_regex('(\d+)', '.*'), output)
-        if match is None or match.group(1) is None:
-            raise Exception("Could not extract the PID of the child command from its output")
-
-        expected_pid = int(match.group(1))
-
-        controllers = os.listdir(self.base_cgroups)
-        for c in controllers:
-            procs = os.path.join(self.base_cgroups, c, "WALinuxAgent", self.ext_handler.name, "cgroup.procs")
-            with open(procs, "r") as f:
-                contents = f.read()
-                pid = int(contents)
-
-                self.assertNotEqual(os.getpid(), pid, "The PID {0} added to {1} was of the launch command caller, not the command itself.".format(pid, procs))
-                self.assertEquals(pid, expected_pid, "The PID of the command was not added to {0}. Expected: {1}, got: {2}".format(procs, expected_pid, pid))
+            # This check will fail if any helper environment variables are added/removed later on
+            self.assertEqual(helper_env_vars, without_os_env)
 
+            # This check is checking if the expected values are set for the extension commands
+            for helper_var in helper_env_vars:
+                self.assertIn("%s=%s" % (helper_var, helper_env_vars[helper_var]), output)
