@@ -64,6 +64,14 @@ DATA_FILE_EXT_DELETION["manifest"] = "wire/manifest_deletion.xml"
 DATA_FILE_EXT_SINGLE = DATA_FILE.copy()
 DATA_FILE_EXT_SINGLE["manifest"] = "wire/manifest_deletion.xml"
 
+DATA_FILE_MULTIPLE_EXT = DATA_FILE.copy()
+DATA_FILE_MULTIPLE_EXT["ext_conf"] = "wire/ext_conf_multiple_extensions.xml"
+
+DATA_FILE_NO_CERT_FORMAT = DATA_FILE.copy()
+DATA_FILE_NO_CERT_FORMAT["certs"] = "wire/certs_no_format_specified.xml"
+
+DATA_FILE_CERT_FORMAT_NOT_PFX = DATA_FILE.copy()
+DATA_FILE_CERT_FORMAT_NOT_PFX["certs"] = "wire/certs_format_not_pfx.xml"
 
 class WireProtocolData(object):
     def __init__(self, data_files=DATA_FILE):
