@@ -76,7 +76,25 @@ class TestHostPlugin(AgentTestCase):
         status_blob.vm_status = restapi.VMStatus(message="Ready", status="Ready")
         return status_blob
 
+    def _relax_timestamp(self, headers):
+        new_headers = []
+
+        for header in headers:
+            header_value = header['headerValue']
+            if header['headerName'] == 'x-ms-date':
+                timestamp = header['headerValue']
+                header_value = timestamp[:timestamp.rfind(":")]
+
+            new_header = {header['headerName']: header_value}
+            new_headers.append(new_header)
+
+        return new_headers
+
     def _compare_data(self, actual, expected):
+        # Remove seconds from the timestamps for testing purposes, that level or granularity introduces test flakiness
+        actual['headers'] = self._relax_timestamp(actual['headers'])
+        expected['headers'] = self._relax_timestamp(expected['headers'])
+
         for k in iter(expected.keys()):
             if k == 'content' or k == 'requestUri':
                 if actual[k] != expected[k]:
@@ -660,11 +678,8 @@ class TestHostPlugin(AgentTestCase):
         # put status blob
         patch_http_put.return_value = MockResponse(None, 500)
 
-        if sys.version_info < (2, 7):
-            self.assertRaises(HttpError, host_plugin.put_vm_status, status_blob, sas_url)
-        else:
-            with self.assertRaises(HttpError):
-                host_plugin.put_vm_status(status_blob=status_blob, sas_url=sas_url)
+        with self.assertRaises(HttpError):
+            host_plugin.put_vm_status(status_blob=status_blob, sas_url=sas_url)
 
         self.assertEqual(1, patch_http_get.call_count)
         self.assertEqual(hostplugin_versions_url, patch_http_get.call_args[0][0])
@@ -703,11 +718,8 @@ class TestHostPlugin(AgentTestCase):
 
         host_plugin.status_error_state.is_triggered = Mock(return_value=True)
 
-        if sys.version_info < (2, 7):
-            self.assertRaises(HttpError, host_plugin.put_vm_status, status_blob, sas_url)
-        else:
-            with self.assertRaises(HttpError):
-                host_plugin.put_vm_status(status_blob=status_blob, sas_url=sas_url)
+        with self.assertRaises(HttpError):
+            host_plugin.put_vm_status(status_blob=status_blob, sas_url=sas_url)
 
         self.assertEqual(1, patch_http_get.call_count)
         self.assertEqual(hostplugin_versions_url, patch_http_get.call_args[0][0])
