@@ -36,6 +36,7 @@ Extensions.Enabled = True
 HttpProxy.Host = None
 HttpProxy.Port = None
 Lib.Dir = /var/lib/waagent
+Logs.Console = True
 Logs.Verbose = False
 OS.AllowHTTP = False
 OS.CheckRdmaDriver = False
@@ -51,18 +52,18 @@ OS.SshDir = /notareal/path
 OS.SudoersDir = /etc/sudoers.d
 OS.UpdateRdmaDriver = False
 Pid.File = /var/run/waagent.pid
+Provisioning.Agent = auto
 Provisioning.AllowResetSysUser = False
 Provisioning.DecodeCustomData = False
 Provisioning.DeleteRootPassword = True
-Provisioning.Enabled = True
 Provisioning.ExecuteCustomData = False
 Provisioning.MonitorHostName = True
 Provisioning.PasswordCryptId = 6
 Provisioning.PasswordCryptSaltLength = 10
 Provisioning.RegenerateSshHostKeyPair = True
 Provisioning.SshHostKeyPairType = rsa
-Provisioning.UseCloudInit = True
 ResourceDisk.EnableSwap = False
+ResourceDisk.EnableSwapEncryption = False
 ResourceDisk.Filesystem = ext4
 ResourceDisk.Format = True
 ResourceDisk.MountOptions = None
@@ -73,13 +74,13 @@ class TestAgent(AgentTestCase):
 
     def test_accepts_configuration_path(self):
         conf_path = os.path.join(data_dir, "test_waagent.conf")
-        c, f, v, cfp = parse_args(["-configuration-path:" + conf_path])
+        c, f, v, d, cfp = parse_args(["-configuration-path:" + conf_path])
         self.assertEqual(cfp, conf_path)
 
     @patch("os.path.exists", return_value=True)
     def test_checks_configuration_path(self, mock_exists):
         conf_path = "/foo/bar-baz/something.conf"
-        c, f, v, cfp = parse_args(["-configuration-path:"+conf_path])
+        c, f, v, d, cfp = parse_args(["-configuration-path:"+conf_path])
         self.assertEqual(cfp, conf_path)
         self.assertEqual(mock_exists.call_count, 1)
 
@@ -88,13 +89,13 @@ class TestAgent(AgentTestCase):
     @patch("sys.exit", side_effect=Exception)
     def test_rejects_missing_configuration_path(self, mock_exit, mock_exists, mock_stderr):
         try:
-            c, f, v, cfp = parse_args(["-configuration-path:/foo/bar.conf"])
+            c, f, v, d, cfp = parse_args(["-configuration-path:/foo/bar.conf"])
             self.assertTrue(False)
         except Exception:
             self.assertEqual(mock_exit.call_count, 1)
 
     def test_configuration_path_defaults_to_none(self):
-        c, f, v, cfp = parse_args([])
+        c, f, v, d, cfp = parse_args([])
         self.assertEqual(cfp, None)
 
     def test_agent_accepts_configuration_path(self):
